@@ -31,7 +31,7 @@ from PyQt4 import QtCore, QtGui
 
 import logger
 
-import api_v1
+import api_v2
 import config
 import config_dialog
 import common
@@ -58,6 +58,7 @@ class NWApp(QtGui.QApplication):
         self.appdir = os.path.join(xdg_config, 'new-wallpaper')
         self.wallsdir = os.path.join(self.appdir, 'wallpapers')
         self.thumbsdir = os.path.join(self.appdir, 'thumbs')
+        self.thumbs256dir = os.path.join(self.appdir, 'thumbs256')
         self.scriptsdir = os.path.join(self.appdir, 'scripts')
 
         for path in (
@@ -65,6 +66,7 @@ class NWApp(QtGui.QApplication):
                 self.appdir,
                 self.wallsdir,
                 self.thumbsdir,
+                self.thumbs256dir,
                 self.scriptsdir
             ):
                 if not os.path.isdir(path):
@@ -196,8 +198,8 @@ class NWApp(QtGui.QApplication):
                     self.bad_urls.extend(result['bad_urls'])
                     self.tray_icon.set_bad_urls(True)
 
-                
-                return self.run_script(filename)
+                result['copypath'] = filename
+                return self.set_desktop_wallpaper(result)
 
             result['bad_urls'].append(result['urls'][result['url_index']])
             result['url_index'] += 1
@@ -214,7 +216,7 @@ class NWApp(QtGui.QApplication):
 
         result['copypath'] = os.path.join(self.wallsdir, result['name'])
         if self.check_sha1(result['copypath']):
-            return self.run_script(result['copypath'])
+            return self.set_desktop_wallpaper(result)
 
         if not result['urls']:
             return self.notify_error(_('There are no urls for this wallpaper'))
@@ -240,15 +242,26 @@ class NWApp(QtGui.QApplication):
 
         return parts[0] == sha1
 
+    def set_desktop_wallpaper(self, result):
+        if not self.run_script(result['copypath']):
+            return
+
+        self.last_set_wallpaper = result
+        self.tray_icon.set_last_set_wallpaper(True)
+
     def run_script(self, wallpaper_path):
         if not self.settings['script/path']:
-            return self.notify_error(_('You must configure script'))
+            self.notify_error(_('You must configure script'))
+            return False
         
         try:
             subprocess.Popen([self.settings['script/path'], wallpaper_path])
         except OSError, exc:
             self.notify_error(
                 _('Script error: ') + exc.strerror.decode('utf8', 'replace'))
+            return False
+
+        return True
 
     def process_results(self, response, params, set_wallpaper=True,
         useronly=True, check_duplicates=True):
@@ -265,7 +278,7 @@ class NWApp(QtGui.QApplication):
                 check_duplicates=False
             )
 
-        results = response['results']
+        results = response['result']
         if not results:
             self.notify_info(_('Not returned any result'))
             return
@@ -297,7 +310,7 @@ class NWApp(QtGui.QApplication):
             if originals:
                 userid, password, secure = self.get_userid_and_password()
 
-                request = api_v1.WallpapersInfo(wallpapers, useronly,
+                request = api_v2.WallpapersInfo(wallpapers, useronly,
                     userid, password, secure)
                 request.success.connect(success_callback)
                 request.error.connect(self.error_callback)
@@ -337,10 +350,12 @@ class NWApp(QtGui.QApplication):
         if self.settings['random_wallpaper/ask_settings']:
             values = {
                 'anonymous': self.settings['auth/anonymous'],
-                'count': self.config.read_int(
-                    'last_random_wallpaper/count', 1),
+                'number': self.config.read_int(
+                    'last_random_wallpaper/number', 1),
                 'resolutions': self.config.read_string(
                     'last_random_wallpaper/resolutions'),
+                'reslist': self.config.read_string(
+                    'last_random_wallpaper/reslist'),
                 'tags': self.config.read_string(
                     'last_random_wallpaper/tags'),
                 'notags': self.config.read_string(
@@ -360,8 +375,9 @@ class NWApp(QtGui.QApplication):
             self.random_wallpaper_dialog.show()
         else:
             self.request_random_wallpaper(
-                self.settings['random_wallpaper/count'],
+                self.settings['random_wallpaper/number'],
                 self.settings['random_wallpaper/resolutions'],
+                self.settings['random_wallpaper/reslist'],
                 self.settings['random_wallpaper/tags'],
                 self.settings['random_wallpaper/notags'],
                 self.settings['random_wallpaper/useronly'],
@@ -371,9 +387,10 @@ class NWApp(QtGui.QApplication):
     @logger.log_exc
     def random_wallpaper_dialog_accepted(self):
         values = self.random_wallpaper_dialog.export_values()
-        self.config.write('last_random_wallpaper/count', values['count'])
+        self.config.write('last_random_wallpaper/number', values['number'])
         self.config.write('last_random_wallpaper/resolutions',
             values['resolutions'])
+        self.config.write('last_random_wallpaper/reslist', values['reslist'])
         self.config.write('last_random_wallpaper/tags', values['tags'])
         self.config.write('last_random_wallpaper/notags', values['notags'])
         self.config.write('last_random_wallpaper/useronly', values['useronly'])
@@ -387,8 +404,8 @@ class NWApp(QtGui.QApplication):
     def random_wallpaper_dialog_rejected(self):
         self.random_wallpaper_dialog.deleteLater()
 
-    def request_random_wallpaper(self, count=1, resolutions=None, tags=None,
-        notags=None, useronly=False, set_wallpaper=True):
+    def request_random_wallpaper(self, number=1, resolutions=None, reslist='',
+        tags=None, notags=None, useronly=False, set_wallpaper=True):
 
         @logger.log_exc
         def success_callback(response):
@@ -399,8 +416,8 @@ class NWApp(QtGui.QApplication):
                 {
                     'title': _('Random wallpaper'),
                     'method': self.request_random_wallpaper,
-                    'args': [count, resolutions, tags, notags, useronly,
-                        set_wallpaper]
+                    'args': [number, resolutions, reslist, tags, notags,
+                        useronly, set_wallpaper]
                 },
                 set_wallpaper,
                 useronly
@@ -408,8 +425,8 @@ class NWApp(QtGui.QApplication):
 
         userid, password, secure = self.get_userid_and_password()
 
-        request = api_v1.WallpapersRandom(count, resolutions, tags, notags,
-            useronly, userid, password, secure)
+        request = api_v2.WallpapersRandom(number, resolutions, reslist, tags,
+            notags, useronly, userid, password, secure)
         request.success.connect(success_callback)
         request.error.connect(self.error_callback)
 
@@ -419,8 +436,8 @@ class NWApp(QtGui.QApplication):
     def last_wallpaper_request(self, checked=False):
         if self.settings['last_wallpaper/ask_settings']:
             values = {
-                'count': self.config.read_int(
-                    'last_last_wallpaper/count', 1),
+                'number': self.config.read_int(
+                    'last_last_wallpaper/number', 1),
                 'set_wallpaper': self.config.read_bool(
                     'last_last_wallpaper/set_wallpaper', True)
             }
@@ -434,14 +451,14 @@ class NWApp(QtGui.QApplication):
             self.last_wallpaper_dialog.show()
         else:
             self.request_last_wallpaper(
-                self.settings['last_wallpaper/count'],
+                self.settings['last_wallpaper/number'],
                 self.settings['last_wallpaper/set_wallpaper']
             )
 
     @logger.log_exc
     def last_wallpaper_dialog_accepted(self):
         values = self.last_wallpaper_dialog.export_values()
-        self.config.write('last_last_wallpaper/count', values['count'])
+        self.config.write('last_last_wallpaper/number', values['number'])
         self.config.write('last_last_wallpaper/set_wallpaper',
             values['set_wallpaper'])
         self.config.sync()
@@ -452,7 +469,7 @@ class NWApp(QtGui.QApplication):
     def last_wallpaper_dialog_rejected(self):
         self.last_wallpaper_dialog.deleteLater()
 
-    def request_last_wallpaper(self, count=1, set_wallpaper=True):
+    def request_last_wallpaper(self, number=1, set_wallpaper=True):
         @logger.log_exc
         def success_callback(response):
             self.set_busy(False)
@@ -462,14 +479,14 @@ class NWApp(QtGui.QApplication):
                 {
                     'title': _('Last wallpaper'),
                     'method': self.request_last_wallpaper,
-                    'args': [count, set_wallpaper]
+                    'args': [number, set_wallpaper]
                 },
                 set_wallpaper
             )
 
         userid, password, secure = self.get_userid_and_password()
 
-        request = api_v1.WallpapersLast(count, userid, password, secure)
+        request = api_v2.WallpapersLast(number, userid, password, secure)
         request.success.connect(success_callback)
         request.error.connect(self.error_callback)
 
@@ -479,8 +496,8 @@ class NWApp(QtGui.QApplication):
     def new_wallpaper_request(self, checked=False):
         if self.settings['new_wallpaper/ask_settings']:
             values = {
-                'count': self.config.read_int(
-                    'last_new_wallpaper/count', 1),
+                'number': self.config.read_int(
+                    'last_new_wallpaper/number', 1),
                 'weight': self.config.read_float(
                     'last_new_wallpaper/weight', 1.0),
                 'set_wallpaper': self.config.read_bool(
@@ -496,7 +513,7 @@ class NWApp(QtGui.QApplication):
             self.new_wallpaper_dialog.show()
         else:
             self.request_new_wallpaper(
-                self.settings['new_wallpaper/count'],
+                self.settings['new_wallpaper/number'],
                 self.settings['new_wallpaper/weight'],
                 self.settings['new_wallpaper/set_wallpaper']
             )
@@ -504,7 +521,7 @@ class NWApp(QtGui.QApplication):
     @logger.log_exc
     def new_wallpaper_dialog_accepted(self):
         values = self.new_wallpaper_dialog.export_values()
-        self.config.write('last_new_wallpaper/count', values['count'])
+        self.config.write('last_new_wallpaper/number', values['number'])
         self.config.write('last_new_wallpaper/weight', values['weight'])
         self.config.write('last_new_wallpaper/set_wallpaper',
             values['set_wallpaper'])
@@ -516,7 +533,7 @@ class NWApp(QtGui.QApplication):
     def new_wallpaper_dialog_rejected(self):
         self.new_wallpaper_dialog.deleteLater()
 
-    def request_new_wallpaper(self, count=1, weight=1.0, set_wallpaper=True):
+    def request_new_wallpaper(self, number=1, weight=1.0, set_wallpaper=True):
         @logger.log_exc
         def success_callback(response):
             self.set_busy(False)
@@ -526,7 +543,7 @@ class NWApp(QtGui.QApplication):
                 {
                     'title': _('New wallpaper'),
                     'method': self.request_new_wallpaper,
-                    'args': [count, weight, set_wallpaper]
+                    'args': [number, weight, set_wallpaper]
                 },
                 set_wallpaper,
                 useronly=False
@@ -534,7 +551,7 @@ class NWApp(QtGui.QApplication):
 
         userid, password, secure = self.get_userid_and_password()
 
-        request = api_v1.WallpapersNew(count, weight, userid, password, secure)
+        request = api_v2.WallpapersNew(number, weight, userid, password, secure)
         request.success.connect(success_callback)
         request.error.connect(self.error_callback)
 
@@ -568,15 +585,12 @@ class NWApp(QtGui.QApplication):
         @logger.log_exc
         def success_callback(response):
             self.set_busy(False)
-            
-            if 'result' not in response:
-                return self.notify_error(_('Bad response'))
 
             self.notify_info(response['result'])
 
         userid, password, secure = self.get_userid_and_password()
 
-        request = api_v1.AddUrl(url, userid, password, tags, xmpp, secure)
+        request = api_v2.AddUrl(url, userid, password, tags, xmpp, secure)
         request.success.connect(success_callback)
         request.error.connect(self.error_callback)
 
@@ -589,7 +603,7 @@ class NWApp(QtGui.QApplication):
         self.request_add(wallpaper, tags, tags_only=True)
 
     def request_add(self, wallpaper, tags, tags_only=False):
-        Request = (api_v1.AddWallpaper, api_v1.AddTags)[tags_only]
+        Request = (api_v2.AddWallpaper, api_v2.AddTags)[tags_only]
         
         userid, password, secure = self.get_userid_and_password()
         
@@ -603,9 +617,6 @@ class NWApp(QtGui.QApplication):
     def add_wallpaper_tags_callback(self, response):
         self.set_busy(False)
 
-        if 'result' not in response:
-            return self.notify_error(_('Bad response'))
-
         result = response['result']
         self.results_dialog.update_result(result)
         self.update_last_set_wallpaper(result)
@@ -623,15 +634,12 @@ class NWApp(QtGui.QApplication):
         def success_callback(response):
             self.set_busy(False)
 
-            if 'result' not in response:
-                return self.notify_error(_('Bad response'))
-
             self.results_dialog.update_result(response['result'])
             self.update_last_set_wallpaper(response['result'])
 
         userid, password, secure = self.get_userid_and_password()
 
-        request = api_v1.RejectWallpaper(wallpaper, userid, password, secure)
+        request = api_v2.RejectWallpaper(wallpaper, userid, password, secure)
         request.success.connect(success_callback)
         request.error.connect(self.error_callback)
 
@@ -644,13 +652,20 @@ class NWApp(QtGui.QApplication):
     @logger.log_exc
     def last_set_wallpaper_triggered(self, checked=False):
         self.last_set_wallpaper['thumbpath'] = os.path.join(
-            self.thumbsdir, self.last_set_wallpaper['name'])
+            (
+                self.thumbsdir,
+                self.thumbs256dir
+            )[self.settings['results/large_thumbs']],
+            self.last_set_wallpaper['name'])
 
         if os.path.isfile(self.last_set_wallpaper['thumbpath']):
             return self.show_last_set_wallpaper()
 
         common.download_url(
-            self.last_set_wallpaper['thumb'],
+            (
+                self.last_set_wallpaper['thumb'],
+                self.last_set_wallpaper['thumb256']
+            )[self.settings['results/large_thumbs']],
             self.last_set_wallpaper['thumbpath'],
             self.download_thumb_callback
         )
@@ -662,7 +677,7 @@ class NWApp(QtGui.QApplication):
         self.set_busy(False)
 
         if not filename:
-            return self.notify_error(_('Thumb downloading error'))
+            return self.notify_error(_('Thumbnail downloading error'))
 
         self.show_last_set_wallpaper()
 
@@ -714,9 +729,6 @@ class NWApp(QtGui.QApplication):
         def success_callback(response):
             self.set_busy(False)
 
-            if 'result' not in response:
-                return self.notify_error(_('Bad response'))
-
             self.notify_info(ngettext(
                 '%d url accepted',
                 '%d urls accepted',
@@ -724,7 +736,7 @@ class NWApp(QtGui.QApplication):
 
         userid, password, secure = self.get_userid_and_password()
 
-        response = api_v1.SendBadUrls(urls, userid, password, secure)
+        response = api_v2.SendBadUrls(urls, userid, password, secure)
         response.success.connect(success_callback)
         response.error.connect(self.error_callback)
 
@@ -772,7 +784,7 @@ class NWApp(QtGui.QApplication):
                 original = choice[0] if choice[4] == 1 else choice[1]
                 copy     = choice[1] if choice[4] == 1 else choice[0]
 
-                request = api_v1.SendDuplicate(original, copy,
+                request = api_v2.SendDuplicate(original, copy,
                     userid, password, secure)
                 request.success.connect(success_callback)
                 request.error.connect(self.error_callback)
@@ -857,8 +869,9 @@ class NWApp(QtGui.QApplication):
     def scheduler_request(self):
         if self.settings['schedule/request'] == 'random_wallpaper':
             self.request_random_wallpaper(
-                self.settings['scheduled_random_wallpaper/count'],
+                self.settings['scheduled_random_wallpaper/number'],
                 self.settings['scheduled_random_wallpaper/resolutions'],
+                self.settings['scheduled_random_wallpaper/reslist'],
                 self.settings['scheduled_random_wallpaper/tags'],
                 self.settings['scheduled_random_wallpaper/notags'],
                 self.settings['scheduled_random_wallpaper/useronly'],
@@ -866,12 +879,12 @@ class NWApp(QtGui.QApplication):
             )
         elif self.settings['schedule/request'] == 'last_wallpaper':
             self.request_last_wallpaper(
-                self.settings['scheduled_last_wallpaper/count'],
+                self.settings['scheduled_last_wallpaper/number'],
                 self.settings['scheduled_last_wallpaper/set_wallpaper']
             )
         elif self.settings['schedule/request'] == 'new_wallpaper':
             self.request_new_wallpaper(
-                self.settings['scheduled_new_wallpaper/count'],
+                self.settings['scheduled_new_wallpaper/number'],
                 self.settings['scheduled_new_wallpaper/weight'],
                 self.settings['scheduled_new_wallpaper/set_wallpaper']
             )
